@@ -3,7 +3,7 @@ use warnings;
 
 package Oh;
 
-our $VERSION = 0.02;
+our $VERSION = 0.03;
 
 use subs qw/error env listp evaluate_element evaluate_list evaluate_function evaluate_builtin evaluate_macro evaluate_lambda find search progn read_element read_string read_list reft/;
 
@@ -667,6 +667,10 @@ sub process_args
         {
           interpret_file shift @commands;
         }
+        elsif ($command eq 'v')
+        {
+          print "oh $VERSION\n";
+        }
         elsif ($command eq 'i')
         {
           repl;
@@ -893,6 +897,18 @@ op
   }
 } 'unless';
 
+op
+{
+  if (evaluate_element shift)
+  {
+    progn @_;
+  }
+  else
+  {
+    undef;
+  }
+} 'when';
+
 op
 {
   my $result;
@@ -922,6 +938,20 @@ op
   evaluate_element $catch if $@;
 } 'try';
 
+op
+{
+  error 'hash did not receive a number of arguments multiple of two' if @_ % 2;
+  my %hash;
+  while (@_)
+  {
+    my ($key, $value) = splice @_, 0, 2;
+    $hash{$key} = evaluate_element $value;
+  }
+  \%hash;
+} 'hash';
+    
+fun { listp ref $_[0] } 'listp';
+fun { my ($obj, $method, @args) = @_; $obj->$method(@args) } 'method';
 
 
 #funs
@@ -988,9 +1018,14 @@ fun
   $string;
 } 'substitute-matches';
 
+fun
+{
+  error 'make-hash did not receive a number of arguments multiple of two' if @_ % 2;
+  +{ @_ }; 
+} 'make-hash';
 fun { $_[0] eq $_[1] } 'eq';
 fun { progn(@_) } 'eval';
-#fun { \@_ } 'list';
+fun { \@_ } 'list';
 fun { pr(@_) } 'pr';
 fun { pr("@_\n") } 'print';
 fun { $_[0]->[0] } 'car';
@@ -1000,25 +1035,31 @@ fun
   shift @list;
   \@list;
 } 'cdr';
-fun { +{ @_ } } 'hash';
-fun { listp ref $_[0] } 'listp';
-fun { my ($obj, $method, @args) = @_; $obj->$method(@args) } 'method';
+
 
 interpret_string <<'oh';
-(macro fun (name args . code) `(sub ,name (scheme-args ,args) ,@code))
-(macro defun (name args . code) `(sub ,name (lisp-args ,args) ,@code))
-(macro lambda (args . code) `(anon (lisp-args ,args) ,@code))
-(macro perl-fun (module name args . code) `(perl-sub ,module ,name (scheme-args ,args) ,@code))
-(macro perl-defun (module name args . code) `(perl-sub ,module ,name (lisp-args ,args) ,@code))
+(macro fun (name args . code)
+ `(sub ,name (scheme-args ,args) ,@code))
+
+(macro defun (name args . code)
+ `(sub ,name (lisp-args ,args) ,@code))
+
+(macro lambda (args . code)
+ `(anon (lisp-args ,args) ,@code))
+
+(macro perl-fun (module name args . code)
+ `(perl-sub ,module ,name (scheme-args ,args) ,@code))
+
+(macro perl-defun (module name args . code)
+ `(perl-sub ,module ,name (lisp-args ,args) ,@code))
+
 (macro define (name . value)
   (if (listp name)
    `(fun ,(car name) ,(cdr name) ,@value)
    `(setq ,name ,(car value))))
-(fun list x x)
+
 oh
 
-
-
   
 
 
